#ifndef INCLUDED_ChaseDemo_h_
#define INCLUDED_ChaseDemo_h_

#include "Behaviors/StateNode.h"
#include "Behaviors/Nodes/SoundNode.h"
#include "Behaviors/Transition.h"
#include "Behaviors/Transitions/EventTrans.h"
#include "Events/EventRouter.h"

//nodes
#include "WaitStateNode.h"
#include "SearchStateNode.h"
#include "TrackStateNode.h"
#include "KickStateNode.h"

//namespace (for event EGIDs)
#include "SoccerNS.h"

class ChaseDemo : public StateNode 
{
  private:
	StateNode* start;

  public:
	ChaseDemo() : StateNode("ChaseDemo"), start(NULL) {}

	virtual void setup() 
	{
		//build state machine

		//create nodes
		StateNode *wait           = addNode(new WaitStateNode());
		StateNode *search = start = addNode(new SearchStateNode()); //start off in the search state
		StateNode *track          = addNode(new TrackStateNode());
		StateNode *kick           = addNode(new KickStateNode());

		//create transitions
		wait->addTransition(new EventTrans(search, EventBase::soccerEGID, SoccerNS::GoSearch));
		search->addTransition(new EventTrans(track, EventBase::soccerEGID, SoccerNS::GoTrack));
		track->addTransition(new EventTrans(kick, EventBase::soccerEGID, SoccerNS::GoKick));
		kick->addTransition(new EventTrans(wait, EventBase::soccerEGID, SoccerNS::GoWait));

		StateNode::setup();
	}

	virtual void teardown() 
	{
		erouter->removeListener(this);
		StateNode::teardown();
	}

	virtual void DoStart() 
	{
		StateNode::DoStart();
		start->DoStart(); //begin the state machine loop by starting the first node
	}

	virtual void DoStop()
	{
		StateNode::DoStop();
	}

  protected:
	ChaseDemo(const ChaseDemo&);
	ChaseDemo& operator=(const ChaseDemo&);
};

#endif
