#ifndef INCLUDED_KickStateNode_h_
#define INCLUDED_KickStateNode_h_

#include "Behaviors/StateNode.h"
#include "Behaviors/Nodes/SoundNode.h"
#include "Behaviors/Transition.h"
#include "Behaviors/Transitions/EventTrans.h"
#include "Events/EventRouter.h"
#include "Shared/ERS7Info.h"

#include "SoundPlay/SoundManager.h"
#include "Motion/MotionManager.h"
#include "Motion/MotionSequenceMC.h"
#include "Shared/WorldState.h"
#include "Shared/SharedObject.h"

#include "SoccerNS.h"

class KickStateNode : public StateNode 
{
  public:
	KickStateNode() : StateNode("KickStateNode"), kick_id(MotionManager::invalid_MC_ID) {}

	virtual void setup() 
	{
		StateNode::setup();
	}

	virtual void teardown() 
	{
		StateNode::teardown();	
	}

	virtual void DoStart() 
	{
		StateNode::DoStart();
		cout << "KickStateNode: in DoStart()" << endl;
		erouter->addListener(this,EventBase::sensorEGID);
	}

	virtual void DoStop() 
	{
		cout << "KickStateNode: in DoStop()" << endl;
		erouter->removeListener(this);
		StateNode::DoStop();
	}

	virtual void processEvent(const EventBase &e)
	{
	
		if(e.getGeneratorID() == EventBase::sensorEGID && kick_id == MotionManager::invalid_MC_ID)
		{
			cout << "KickStateNode: Starting kick" << endl;
			
			//load the kick, play the motion, and create a listener for when the kick finishes
			std::string kickmot = config->motion.makePath("my_k.mot");
			SharedObject< MotionSequenceMC< MotionSequence::SizeLarge > >kick(kickmot.c_str());
			kick_id = motman->addPrunableMotion(kick, MotionManager::kStdPriority);
			erouter->addListener(this,EventBase::motmanEGID,kick_id,EventBase::deactivateETID);
		}
		else if( e.getGeneratorID() == EventBase::motmanEGID && e.getSourceID() == kick_id && e.getTypeID() == EventBase::deactivateETID ) 
		{	
			//kick is finished, stop moving, remove listener, and post event
			kick_id = MotionManager::invalid_MC_ID;
			erouter->removeListener(this,EventBase::motmanEGID,kick_id,EventBase::deactivateETID);

			//post event
			cout << "KickStateNode: Successful kick, posting kick->wait event" << endl;
			erouter->postEvent(EventBase::soccerEGID, SoccerNS::GoWait, EventBase::statusETID, 0);
		}
	}

  protected:
	KickStateNode(const KickStateNode&);
 	KickStateNode& operator=(const KickStateNode&);
	MotionManager::MC_ID kick_id;

};

#endif
