#include "PenaltyGoalie.h"
#include "Events/EventRouter.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/WorldState.h"
#include "Motion/HeadPointerMC.h"
#include "Motion/WalkMC.h"
#include "Shared/ProjectInterface.h"

inline double DtoR(double deg) { return (deg/180.0*M_PI); }

const float MAXAREA = 0.04;

const int READYTIMER = 5555;
const int PANTIMER = 6666;

void PenaltyGoalie::DoStart()
{
	BehaviorBase::DoStart();
	headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
	walker_id = motman->addPersistentMotion(SharedObject<WalkMC>());
	erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visPinkBallSID);

	savedball = false;
	erouter->addTimer(this,PANTIMER,5000,true); //5 second timer, repeat

	MMAccessor<WalkMC> walk(walker_id);
	walk.mc()->setTargetVelocity(180.0,0,0,5);
	HeadPointerMC * headpointer = (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
	headpointer->setJoints(0,0,0);
	motman->checkinMotion(headpointer_id);
}

void PenaltyGoalie::DoStop() 
{
       erouter->removeListener(this);
       motman->removeMotion(headpointer_id);
       motman->removeMotion(walker_id);
       BehaviorBase::DoStop();
}

void PenaltyGoalie::processEvent(const EventBase& event) 
{
	if(event.getSourceID() == READYTIMER)
	{
		//saved ball already, stand up and wait for another ball
		cout << "Stand up again" << endl;
		SharedObject<MediumMotionSequenceMC>  s("/ms/data/motion/ready.mot");
		motman->addPrunableMotion(s);
		savedball = false;
	}
	else if(event.getSourceID() == PANTIMER && !savedball)
	{
		//pan the head to see if the ball is to the sides
		SharedObject<MediumMotionSequenceMC>  s("/ms/data/motion/pan.mot");
		motman->addPrunableMotion(s);
	}
	if(event.getSourceID()==ProjectInterface::visPinkBallSID && !savedball)
	{
		//the ball is in view
		float area = static_cast<const VisionObjectEvent*>(&event)->getArea();
		float horz = static_cast<const VisionObjectEvent*>(&event)->getCenterX();
		float vert = static_cast<const VisionObjectEvent*>(&event)->getCenterY();

		cout << "Area: " << area << endl;
		//look at ball
		double tilt=state->outputs[HeadOffset+TiltOffset]-vert*M_PI/7.5;
		double pan=state->outputs[HeadOffset+PanOffset]-horz*M_PI/6;
		if(tilt<DtoR(-20))
			tilt=DtoR(-20);
		if(tilt>DtoR(40))
			tilt=DtoR(40);
		if(pan>DtoR(80))
			pan=DtoR(80);
		if(pan<DtoR(-80))
			pan=DtoR(-80);
		HeadPointerMC * headpointer = (HeadPointerMC*)motman->checkoutMotion(headpointer_id);
		headpointer->setJoints(tilt/2,pan/2,0);
		motman->checkinMotion(headpointer_id);

		if(area > MAXAREA)
		{
			cout << "Save!" << endl;
			SharedObject<MediumMotionSequenceMC>  s("/ms/data/motion/save.mot");
			motman->addPrunableMotion(s);
			savedball = true;
			erouter->addTimer(this,READYTIMER,3000,false); //three second timer, no repeat
		}
	}


}