#ifndef INCLUDED_SearchStateNode_h_
#define INCLUDED_SearchStateNode_h_

#include "Behaviors/StateNode.h"
#include "Behaviors/Transition.h"
#include "Behaviors/Transitions/EventTrans.h"
#include "Events/EventRouter.h"
#include "Shared/ERS7Info.h"

#include "Motion/MotionManager.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/WorldState.h"
#include "Motion/HeadPointerMC.h"
#include "Motion/WalkMC.h"
#include "Shared/WMclass.h"
#include "Motion/MMAccessor.h"
#include "Shared/ProjectInterface.h"

#include "SoccerNS.h"

#define MAX_AREA 0.75

class SearchStateNode : public StateNode 
{
  public:
	SearchStateNode() : StateNode("SearchStateNode"), headpointer_id(MotionManager::invalid_MC_ID), walker_id(MotionManager::invalid_MC_ID), saw_ball(false) {}

	virtual void setup() 
	{
		StateNode::setup();
	}

	virtual void teardown() 
	{
		StateNode::teardown();
	}

	virtual void DoStart() 
	{
    	StateNode::DoStart();
		cout << "SearchStateNode: inside DoStart()" << endl;

		saw_ball = false;

		walker_id = motman->addPersistentMotion(SharedObject<WalkMC>());
		headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
    	MMAccessor<HeadPointerMC> head(headpointer_id);
    	head.mc()->setJoints(0,0,0); //look forward

		erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visPinkBallSID);
		erouter->addTimer(this,2222,250,true); //quarter second timer
	}

	virtual void DoStop() 
	{
		cout << "SearchState: inside DoStop()" << endl;
		erouter->removeListener(this);
		motman->removeMotion(headpointer_id);
		headpointer_id = MotionManager::invalid_MC_ID;
		motman->removeMotion(walker_id);
		walker_id = MotionManager::invalid_MC_ID;

		StateNode::DoStop();
	}

	virtual void processEvent(const EventBase &event) 
	{
		if(event.getGeneratorID() == EventBase::timerEGID)
		{
			if(saw_ball)
			{
				//we see the ball, stop searching and start moving forward
				cout << "SearchStateNode: Ball in view, go track (post search->track event)" << endl;
				erouter->postEvent( EventBase::soccerEGID, SoccerNS::GoTrack, EventBase::statusETID, 0);
			}
			else if(!saw_ball)
			{
				//don't see the ball, keep searching
				cout << "SearchStateNode: Searching... ";
				MMAccessor<WalkMC> walk(walker_id);
				walk.mc()->setTargetVelocity(0,0,1.4,2);
			}
		}

		static float area=0;
		const VisionObjectEvent *ve = dynamic_cast<const VisionObjectEvent*>(&event);

		if(ve!=NULL && event.getTypeID()==EventBase::statusETID && event.getSourceID() == ProjectInterface::visPinkBallSID) 
		{
			area=ve->getArea();
			cout << "Area: " << area << endl;

			if(area > .001)
			{
				//we see the ball, get ready to transition
				cout << "SearchStateNode: Saw the ball" << endl;
				saw_ball = true;
				MMAccessor<WalkMC> walk(walker_id);
				walk.mc()->setTargetVelocity(0,0,0);
			}
		} 
	}

  protected:
	SearchStateNode(const SearchStateNode&);
	SearchStateNode& operator=(const SearchStateNode&);

	MotionManager::MC_ID headpointer_id;
	MotionManager::MC_ID walker_id;
	bool saw_ball;

	inline double DtoR(double deg) { return (deg/180.0*M_PI); }

};

#endif
