#ifndef INCLUDED_TrackStateNode_h_
#define INCLUDED_TrackStateNode_h_

#include "Behaviors/StateNode.h"
#include "Behaviors/Transition.h"
#include "Behaviors/Transitions/EventTrans.h"
#include "Events/EventRouter.h"
#include "Shared/ERS7Info.h"

#include "Motion/MotionManager.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/WorldState.h"
#include "Motion/HeadPointerMC.h"
#include "Motion/WalkMC.h"
#include "Shared/WMclass.h"
#include "Motion/MMAccessor.h"
#include "Shared/ProjectInterface.h"

#include "SoccerNS.h"

#define MAX_AREA 0.75

class TrackStateNode : public StateNode 
{
  public:
    TrackStateNode() : StateNode("TrackStateNode"), headpointer_id(MotionManager::invalid_MC_ID), walker_id(MotionManager::invalid_MC_ID), is_stopping(false) {}

    virtual void setup() 
	{
		StateNode::setup();
	}

	virtual void teardown() 
	{
		StateNode::teardown();
	}

	virtual void DoStart() 
	{
		StateNode::DoStart();
		cout << "TrackStateNode: inside DoStart()" << endl;
		
		walker_id = motman->addPersistentMotion(SharedObject<WalkMC>());
		headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
		erouter->addListener(this,EventBase::visObjEGID,ProjectInterface::visPinkBallSID);

		is_stopping = false;

	}

	virtual void DoStop() 
	{
		cout << "TrackState: inside DoStop()" << endl;
		erouter->removeListener(this);
		motman->removeMotion(headpointer_id);
		headpointer_id = MotionManager::invalid_MC_ID;
		motman->removeMotion(walker_id);
		walker_id = MotionManager::invalid_MC_ID;

		StateNode::DoStop();
	}

	virtual void processEvent(const EventBase &event) 
	{
		static float horiz=0, vert=0, area=0;
		const VisionObjectEvent *ve = dynamic_cast<const VisionObjectEvent*>(&event);

		if(ve!=NULL && event.getTypeID()==EventBase::statusETID && event.getSourceID() == ProjectInterface::visPinkBallSID) 
		{
			//see the pink ball, get area
			horiz=ve->getCenterX();
			vert=ve->getCenterY();
			area=ve->getArea();
			cout << "TrackStateNode: Area: " << area << ", is_stopping = " << is_stopping << endl;

			if(!is_stopping)
			{
				//ball is far away, keep tracking the ball and move closer
				double tilt=state->outputs[HeadOffset+TiltOffset]-vert*M_PI/6;
				double pan=state->outputs[HeadOffset+PanOffset]-horiz*M_PI/7.5;
				if(tilt>outputRanges[HeadOffset+TiltOffset][MaxRange])
					tilt=outputRanges[HeadOffset+TiltOffset][MaxRange];
				if(tilt<outputRanges[HeadOffset+TiltOffset][MinRange]*3/4)
					tilt=outputRanges[HeadOffset+TiltOffset][MinRange]*3/4;
				if(pan>outputRanges[HeadOffset+PanOffset][MaxRange]*2/3)
					pan=outputRanges[HeadOffset+PanOffset][MaxRange]*2/3;
				if(pan<outputRanges[HeadOffset+PanOffset][MinRange]*2/3)
					pan=outputRanges[HeadOffset+PanOffset][MinRange]*2/3;

				MMAccessor<HeadPointerMC>(headpointer_id)->setJoints(tilt,pan,0);

				MMAccessor<WalkMC> walker(walker_id);
				if(pan<-.05 || pan>.05)
					walker->setTargetVelocity(100,0,pan);
				else
					walker->setTargetVelocity(160,0,0);
			}
			if(area > MAX_AREA && !is_stopping) 
			{
				//max area has been reached, walk for a little bit to get in position for the kick
				cout << "TrackStateNode: Saw max area, walk for a little more" << endl;
				erouter->addTimer(this,1111,750,false); //.75 second, non reoccuring timer
				MMAccessor<WalkMC> walker(walker_id);
				walker->setTargetVelocity(100,0,0); 
				is_stopping = 1;
			}
		} 
		else if(event.getGeneratorID() == EventBase::timerEGID && is_stopping)
		{
			//the dog is now positioned in front of the ball, post the kick event!
			cout << "TrackStateNode: Walked for one second, now posting the track -> kick event" << endl;
			erouter->postEvent( EventBase::soccerEGID, SoccerNS::GoKick, EventBase::statusETID, 0);
		}
	}

  protected:
	TrackStateNode(const TrackStateNode&);
	TrackStateNode& operator=(const TrackStateNode&);

	MotionManager::MC_ID headpointer_id;
	MotionManager::MC_ID walker_id;
	bool is_stopping;

	inline double DtoR(double deg) { return (deg/180.0*M_PI); }

};

#endif
