#ifndef INCLUDED_WaitStateNode_h_
#define INCLUDED_WaitStateNode_h_

#include "Behaviors/StateNode.h"
#include "Behaviors/Transition.h"
#include "Behaviors/Transitions/EventTrans.h"
#include "Events/EventRouter.h"
#include "Shared/ERS7Info.h"

#include "Motion/MotionManager.h"
#include "Events/VisionObjectEvent.h"
#include "Shared/WorldState.h"
#include "Motion/HeadPointerMC.h"
#include "Motion/WalkMC.h"
#include "Shared/WMclass.h"
#include "Motion/MMAccessor.h"
#include "Shared/ProjectInterface.h"

#include "SoccerNS.h"

class WaitStateNode : public StateNode 
{
  public:
	WaitStateNode() : StateNode("WaitStateNode") {}

	virtual void setup() 
	{
	    StateNode::setup();
	}

	virtual void teardown()
	{
	    StateNode::teardown();
	}

	virtual void DoStart() 
	{
    	StateNode::DoStart();
		cout << "WaitStateNode: inside DoStart" << endl;
		headpointer_id = motman->addPersistentMotion(SharedObject<HeadPointerMC>());
		MMAccessor<HeadPointerMC> head(headpointer_id);
		head.mc()->setJoints(0,0,0);

		erouter->addTimer(this,3333,1250,false); //1.25 seconds
	}

	virtual void DoStop() 
	{
		cout << "WaitStateNode: inside DoStop" << endl;
		erouter->removeListener(this);
		motman->removeMotion(headpointer_id);
		headpointer_id = MotionManager::invalid_MC_ID;
		
		StateNode::DoStop();
	}

	virtual void processEvent(const EventBase &event) 
	{
		if(event.getGeneratorID() == EventBase::timerEGID)
		{
			//timer is up, transition to search node
			cout << "WaitStateNode: waited for a bit, posting wait->search event" << endl;
			erouter->postEvent(EventBase::soccerEGID, SoccerNS::GoSearch, EventBase::statusETID, 0);
		}
	}

  protected:
	WaitStateNode(const WaitStateNode&);
	WaitStateNode& operator=(const WaitStateNode&);

	MotionManager::MC_ID headpointer_id;
};

#endif
